@@ -22,9 +22,11 @@
  *
  */
 
+#include "aco_ir.h"
+
 #include <map>
 #include <unordered_map>
-#include "aco_ir.h"
+#include <vector>
 
 /*
  * Implements the algorithm for dominator-tree value numbering
@@ -34,8 +36,9 @@
 namespace aco {
 namespace {
 
-inline
-uint32_t murmur_32_scramble(uint32_t h, uint32_t k) {
+inline uint32_t
+murmur_32_scramble(uint32_t h, uint32_t k)
+{
    k *= 0xcc9e2d51;
    k = (k << 15) | (k >> 17);
    h ^= k * 0x1b873593;
@@ -44,8 +47,9 @@ uint32_t murmur_32_scramble(uint32_t h, uint32_t k) {
    return h;
 }
 
-template<typename T>
-uint32_t hash_murmur_32(Instruction* instr)
+template <typename T>
+uint32_t
+hash_murmur_32(Instruction* instr)
 {
    uint32_t hash = uint32_t(instr->format) << 16 | uint32_t(instr->opcode);
 
@@ -56,7 +60,7 @@ uint32_t hash_murmur_32(Instruction* instr)
    for (unsigned i = 2; i < (sizeof(T) >> 2); i++) {
       uint32_t u;
       /* Accesses it though a byte array, so doesn't violate the strict aliasing rule */
-      memcpy(&u, reinterpret_cast<uint8_t *>(instr) + i * 4, 4);
+      memcpy(&u, reinterpret_cast<uint8_t*>(instr) + i * 4, 4);
       hash = murmur_32_scramble(hash, u);
    }
 
@@ -81,7 +85,7 @@ struct InstrHash {
    std::size_t operator()(Instruction* instr) const
    {
       if (instr->isVOP3())
-         return hash_murmur_32<VOP3A_instruction>(instr);
+         return hash_murmur_32<VOP3_instruction>(instr);
 
       if (instr->isDPP())
          return hash_murmur_32<DPP_instruction>(instr);
@@ -90,32 +94,19 @@ struct InstrHash {
          return hash_murmur_32<SDWA_instruction>(instr);
 
       switch (instr->format) {
-      case Format::SMEM:
-         return hash_murmur_32<SMEM_instruction>(instr);
-      case Format::VINTRP:
-         return hash_murmur_32<Interp_instruction>(instr);
-      case Format::DS:
-         return hash_murmur_32<DS_instruction>(instr);
-      case Format::SOPP:
-         return hash_murmur_32<SOPP_instruction>(instr);
-      case Format::SOPK:
-         return hash_murmur_32<SOPK_instruction>(instr);
-      case Format::EXP:
-         return hash_murmur_32<Export_instruction>(instr);
-      case Format::MUBUF:
-         return hash_murmur_32<MUBUF_instruction>(instr);
-      case Format::MIMG:
-         return hash_murmur_32<MIMG_instruction>(instr);
-      case Format::MTBUF:
-         return hash_murmur_32<MTBUF_instruction>(instr);
-      case Format::FLAT:
-         return hash_murmur_32<FLAT_instruction>(instr);
-      case Format::PSEUDO_BRANCH:
-         return hash_murmur_32<Pseudo_branch_instruction>(instr);
-      case Format::PSEUDO_REDUCTION:
-         return hash_murmur_32<Pseudo_reduction_instruction>(instr);
-      default:
-         return hash_murmur_32<Instruction>(instr);
+      case Format::SMEM: return hash_murmur_32<SMEM_instruction>(instr);
+      case Format::VINTRP: return hash_murmur_32<Interp_instruction>(instr);
+      case Format::DS: return hash_murmur_32<DS_instruction>(instr);
+      case Format::SOPP: return hash_murmur_32<SOPP_instruction>(instr);
+      case Format::SOPK: return hash_murmur_32<SOPK_instruction>(instr);
+      case Format::EXP: return hash_murmur_32<Export_instruction>(instr);
+      case Format::MUBUF: return hash_murmur_32<MUBUF_instruction>(instr);
+      case Format::MIMG: return hash_murmur_32<MIMG_instruction>(instr);
+      case Format::MTBUF: return hash_murmur_32<MTBUF_instruction>(instr);
+      case Format::FLAT: return hash_murmur_32<FLAT_instruction>(instr);
+      case Format::PSEUDO_BRANCH: return hash_murmur_32<Pseudo_branch_instruction>(instr);
+      case Format::PSEUDO_REDUCTION: return hash_murmur_32<Pseudo_reduction_instruction>(instr);
+      default: return hash_murmur_32<Instruction>(instr);
       }
    }
 };
@@ -127,7 +118,8 @@ struct InstrPred {
          return false;
       if (a->opcode != b->opcode)
          return false;
-      if (a->operands.size() != b->operands.size() || a->definitions.size() != b->definitions.size())
+      if (a->operands.size() != b->operands.size() ||
+          a->definitions.size() != b->definitions.size())
          return false; /* possible with pseudo-instructions */
       for (unsigned i = 0; i < a->operands.size(); i++) {
          if (a->operands[i].isConstant()) {
@@ -135,14 +127,12 @@ struct InstrPred {
                return false;
             if (a->operands[i].constantValue() != b->operands[i].constantValue())
                return false;
-         }
-         else if (a->operands[i].isTemp()) {
+         } else if (a->operands[i].isTemp()) {
             if (!b->operands[i].isTemp())
                return false;
             if (a->operands[i].tempId() != b->operands[i].tempId())
                return false;
-         }
-         else if (a->operands[i].isUndefined() ^ b->operands[i].isUndefined())
+         } else if (a->operands[i].isUndefined() ^ b->operands[i].isUndefined())
             return false;
          if (a->operands[i].isFixed()) {
             if (!b->operands[i].isFixed())
@@ -173,169 +163,114 @@ struct InstrPred {
       if (a->opcode == aco_opcode::v_readfirstlane_b32)
          return a->pass_flags == b->pass_flags;
 
-      /* The results of VOPC depend on the exec mask if used for subgroup operations. */
-      if ((uint32_t) a->format & (uint32_t) Format::VOPC && a->pass_flags != b->pass_flags)
-         return false;
-
       if (a->isVOP3()) {
-         VOP3A_instruction* a3 = static_cast<VOP3A_instruction*>(a);
-         VOP3A_instruction* b3 = static_cast<VOP3A_instruction*>(b);
+         VOP3_instruction& a3 = a->vop3();
+         VOP3_instruction& b3 = b->vop3();
          for (unsigned i = 0; i < 3; i++) {
-            if (a3->abs[i] != b3->abs[i] ||
-                a3->neg[i] != b3->neg[i])
+            if (a3.abs[i] != b3.abs[i] || a3.neg[i] != b3.neg[i])
                return false;
          }
-         return a3->clamp == b3->clamp &&
-                a3->omod == b3->omod &&
-                a3->opsel == b3->opsel;
+         return a3.clamp == b3.clamp && a3.omod == b3.omod && a3.opsel == b3.opsel;
       }
       if (a->isDPP()) {
-         DPP_instruction* aDPP = static_cast<DPP_instruction*>(a);
-         DPP_instruction* bDPP = static_cast<DPP_instruction*>(b);
-         return aDPP->pass_flags == bDPP->pass_flags &&
-                aDPP->dpp_ctrl == bDPP->dpp_ctrl &&
-                aDPP->bank_mask == bDPP->bank_mask &&
-                aDPP->row_mask == bDPP->row_mask &&
-                aDPP->bound_ctrl == bDPP->bound_ctrl &&
-                aDPP->abs[0] == bDPP->abs[0] &&
-                aDPP->abs[1] == bDPP->abs[1] &&
-                aDPP->neg[0] == bDPP->neg[0] &&
-                aDPP->neg[1] == bDPP->neg[1];
+         DPP_instruction& aDPP = a->dpp();
+         DPP_instruction& bDPP = b->dpp();
+         return aDPP.pass_flags == bDPP.pass_flags && aDPP.dpp_ctrl == bDPP.dpp_ctrl &&
+                aDPP.bank_mask == bDPP.bank_mask && aDPP.row_mask == bDPP.row_mask &&
+                aDPP.bound_ctrl == bDPP.bound_ctrl && aDPP.abs[0] == bDPP.abs[0] &&
+                aDPP.abs[1] == bDPP.abs[1] && aDPP.neg[0] == bDPP.neg[0] &&
+                aDPP.neg[1] == bDPP.neg[1];
       }
       if (a->isSDWA()) {
-         SDWA_instruction* aSDWA = static_cast<SDWA_instruction*>(a);
-         SDWA_instruction* bSDWA = static_cast<SDWA_instruction*>(b);
-         return aSDWA->sel[0] == bSDWA->sel[0] &&
-                aSDWA->sel[1] == bSDWA->sel[1] &&
-                aSDWA->dst_sel == bSDWA->dst_sel &&
-                aSDWA->abs[0] == bSDWA->abs[0] &&
-                aSDWA->abs[1] == bSDWA->abs[1] &&
-                aSDWA->neg[0] == bSDWA->neg[0] &&
-                aSDWA->neg[1] == bSDWA->neg[1] &&
-                aSDWA->dst_preserve == bSDWA->dst_preserve &&
-                aSDWA->clamp == bSDWA->clamp &&
-                aSDWA->omod == bSDWA->omod;
+         SDWA_instruction& aSDWA = a->sdwa();
+         SDWA_instruction& bSDWA = b->sdwa();
+         return aSDWA.sel[0] == bSDWA.sel[0] && aSDWA.sel[1] == bSDWA.sel[1] &&
+                aSDWA.dst_sel == bSDWA.dst_sel && aSDWA.abs[0] == bSDWA.abs[0] &&
+                aSDWA.abs[1] == bSDWA.abs[1] && aSDWA.neg[0] == bSDWA.neg[0] &&
+                aSDWA.neg[1] == bSDWA.neg[1] && aSDWA.dst_preserve == bSDWA.dst_preserve &&
+                aSDWA.clamp == bSDWA.clamp && aSDWA.omod == bSDWA.omod;
       }
 
       switch (a->format) {
-         case Format::SOPK: {
-            if (a->opcode == aco_opcode::s_getreg_b32)
-               return false;
-            SOPK_instruction* aK = static_cast<SOPK_instruction*>(a);
-            SOPK_instruction* bK = static_cast<SOPK_instruction*>(b);
-            return aK->imm == bK->imm;
-         }
-         case Format::SMEM: {
-            SMEM_instruction* aS = static_cast<SMEM_instruction*>(a);
-            SMEM_instruction* bS = static_cast<SMEM_instruction*>(b);
-            /* isel shouldn't be creating situations where this assertion fails */
-            assert(aS->prevent_overflow == bS->prevent_overflow);
-            return aS->sync.can_reorder() && bS->sync.can_reorder() &&
-                   aS->sync == bS->sync && aS->glc == bS->glc && aS->dlc == bS->dlc &&
-                   aS->nv == bS->nv && aS->disable_wqm == bS->disable_wqm &&
-                   aS->prevent_overflow == bS->prevent_overflow;
-         }
-         case Format::VINTRP: {
-            Interp_instruction* aI = static_cast<Interp_instruction*>(a);
-            Interp_instruction* bI = static_cast<Interp_instruction*>(b);
-            if (aI->attribute != bI->attribute)
-               return false;
-            if (aI->component != bI->component)
-               return false;
-            return true;
-         }
-         case Format::VOP3P: {
-            VOP3P_instruction* a3P = static_cast<VOP3P_instruction*>(a);
-            VOP3P_instruction* b3P = static_cast<VOP3P_instruction*>(b);
-            for (unsigned i = 0; i < 3; i++) {
-               if (a3P->neg_lo[i] != b3P->neg_lo[i] ||
-                   a3P->neg_hi[i] != b3P->neg_hi[i])
-                  return false;
-            }
-            return a3P->opsel_lo == b3P->opsel_lo &&
-                   a3P->opsel_hi == b3P->opsel_hi &&
-                   a3P->clamp == b3P->clamp;
-         }
-         case Format::PSEUDO_REDUCTION: {
-            Pseudo_reduction_instruction *aR = static_cast<Pseudo_reduction_instruction*>(a);
-            Pseudo_reduction_instruction *bR = static_cast<Pseudo_reduction_instruction*>(b);
-            return aR->pass_flags == bR->pass_flags &&
-                   aR->reduce_op == bR->reduce_op &&
-                   aR->cluster_size == bR->cluster_size;
-         }
-         case Format::MTBUF: {
-            MTBUF_instruction* aM = static_cast<MTBUF_instruction *>(a);
-            MTBUF_instruction* bM = static_cast<MTBUF_instruction *>(b);
-            return aM->sync.can_reorder() && bM->sync.can_reorder() &&
-                   aM->sync == bM->sync &&
-                   aM->dfmt == bM->dfmt &&
-                   aM->nfmt == bM->nfmt &&
-                   aM->offset == bM->offset &&
-                   aM->offen == bM->offen &&
-                   aM->idxen == bM->idxen &&
-                   aM->glc == bM->glc &&
-                   aM->dlc == bM->dlc &&
-                   aM->slc == bM->slc &&
-                   aM->tfe == bM->tfe &&
-                   aM->disable_wqm == bM->disable_wqm;
-         }
-         case Format::MUBUF: {
-            MUBUF_instruction* aM = static_cast<MUBUF_instruction *>(a);
-            MUBUF_instruction* bM = static_cast<MUBUF_instruction *>(b);
-            return aM->sync.can_reorder() && bM->sync.can_reorder() &&
-                   aM->sync == bM->sync &&
-                   aM->offset == bM->offset &&
-                   aM->offen == bM->offen &&
-                   aM->idxen == bM->idxen &&
-                   aM->glc == bM->glc &&
-                   aM->dlc == bM->dlc &&
-                   aM->slc == bM->slc &&
-                   aM->tfe == bM->tfe &&
-                   aM->lds == bM->lds &&
-                   aM->disable_wqm == bM->disable_wqm;
-         }
-         /* we want to optimize these in NIR and don't hassle with load-store dependencies */
-         case Format::FLAT:
-         case Format::GLOBAL:
-         case Format::SCRATCH:
-         case Format::EXP:
-         case Format::SOPP:
-         case Format::PSEUDO_BRANCH:
-         case Format::PSEUDO_BARRIER:
+      case Format::SOPK: {
+         if (a->opcode == aco_opcode::s_getreg_b32)
             return false;
-         case Format::DS: {
-            if (a->opcode != aco_opcode::ds_bpermute_b32 &&
-                a->opcode != aco_opcode::ds_permute_b32 &&
-                a->opcode != aco_opcode::ds_swizzle_b32)
+         SOPK_instruction& aK = a->sopk();
+         SOPK_instruction& bK = b->sopk();
+         return aK.imm == bK.imm;
+      }
+      case Format::SMEM: {
+         SMEM_instruction& aS = a->smem();
+         SMEM_instruction& bS = b->smem();
+         /* isel shouldn't be creating situations where this assertion fails */
+         assert(aS.prevent_overflow == bS.prevent_overflow);
+         return aS.sync == bS.sync && aS.glc == bS.glc && aS.dlc == bS.dlc && aS.nv == bS.nv &&
+                aS.disable_wqm == bS.disable_wqm && aS.prevent_overflow == bS.prevent_overflow;
+      }
+      case Format::VINTRP: {
+         Interp_instruction& aI = a->vintrp();
+         Interp_instruction& bI = b->vintrp();
+         if (aI.attribute != bI.attribute)
+            return false;
+         if (aI.component != bI.component)
+            return false;
+         return true;
+      }
+      case Format::VOP3P: {
+         VOP3P_instruction& a3P = a->vop3p();
+         VOP3P_instruction& b3P = b->vop3p();
+         for (unsigned i = 0; i < 3; i++) {
+            if (a3P.neg_lo[i] != b3P.neg_lo[i] || a3P.neg_hi[i] != b3P.neg_hi[i])
                return false;
-            DS_instruction* aD = static_cast<DS_instruction *>(a);
-            DS_instruction* bD = static_cast<DS_instruction *>(b);
-            return aD->sync.can_reorder() && bD->sync.can_reorder() &&
-                   aD->sync == bD->sync &&
-                   aD->pass_flags == bD->pass_flags &&
-                   aD->gds == bD->gds &&
-                   aD->offset0 == bD->offset0 &&
-                   aD->offset1 == bD->offset1;
          }
-         case Format::MIMG: {
-            MIMG_instruction* aM = static_cast<MIMG_instruction*>(a);
-            MIMG_instruction* bM = static_cast<MIMG_instruction*>(b);
-            return aM->sync.can_reorder() && bM->sync.can_reorder() &&
-                   aM->sync == bM->sync &&
-                   aM->dmask == bM->dmask &&
-                   aM->unrm == bM->unrm &&
-                   aM->glc == bM->glc &&
-                   aM->slc == bM->slc &&
-                   aM->tfe == bM->tfe &&
-                   aM->da == bM->da &&
-                   aM->lwe == bM->lwe &&
-                   aM->r128 == bM->r128 &&
-                   aM->a16 == bM->a16 &&
-                   aM->d16 == bM->d16 &&
-                   aM->disable_wqm == bM->disable_wqm;
-         }
-         default:
-            return true;
+         return a3P.opsel_lo == b3P.opsel_lo && a3P.opsel_hi == b3P.opsel_hi &&
+                a3P.clamp == b3P.clamp;
+      }
+      case Format::PSEUDO_REDUCTION: {
+         Pseudo_reduction_instruction& aR = a->reduction();
+         Pseudo_reduction_instruction& bR = b->reduction();
+         return aR.pass_flags == bR.pass_flags && aR.reduce_op == bR.reduce_op &&
+                aR.cluster_size == bR.cluster_size;
+      }
+      case Format::DS: {
+         assert(a->opcode == aco_opcode::ds_bpermute_b32 ||
+                a->opcode == aco_opcode::ds_permute_b32 || a->opcode == aco_opcode::ds_swizzle_b32);
+         DS_instruction& aD = a->ds();
+         DS_instruction& bD = b->ds();
+         return aD.sync == bD.sync && aD.pass_flags == bD.pass_flags && aD.gds == bD.gds &&
+                aD.offset0 == bD.offset0 && aD.offset1 == bD.offset1;
+      }
+      case Format::MTBUF: {
+         MTBUF_instruction& aM = a->mtbuf();
+         MTBUF_instruction& bM = b->mtbuf();
+         return aM.sync == bM.sync && aM.dfmt == bM.dfmt && aM.nfmt == bM.nfmt &&
+                aM.offset == bM.offset && aM.offen == bM.offen && aM.idxen == bM.idxen &&
+                aM.glc == bM.glc && aM.dlc == bM.dlc && aM.slc == bM.slc && aM.tfe == bM.tfe &&
+                aM.disable_wqm == bM.disable_wqm;
+      }
+      case Format::MUBUF: {
+         MUBUF_instruction& aM = a->mubuf();
+         MUBUF_instruction& bM = b->mubuf();
+         return aM.sync == bM.sync && aM.offset == bM.offset && aM.offen == bM.offen &&
+                aM.idxen == bM.idxen && aM.glc == bM.glc && aM.dlc == bM.dlc && aM.slc == bM.slc &&
+                aM.tfe == bM.tfe && aM.lds == bM.lds && aM.disable_wqm == bM.disable_wqm;
+      }
+      case Format::MIMG: {
+         MIMG_instruction& aM = a->mimg();
+         MIMG_instruction& bM = b->mimg();
+         return aM.sync == bM.sync && aM.dmask == bM.dmask && aM.unrm == bM.unrm &&
+                aM.glc == bM.glc && aM.slc == bM.slc && aM.tfe == bM.tfe && aM.da == bM.da &&
+                aM.lwe == bM.lwe && aM.r128 == bM.r128 && aM.a16 == bM.a16 && aM.d16 == bM.d16 &&
+                aM.disable_wqm == bM.disable_wqm;
+      }
+      case Format::FLAT:
+      case Format::GLOBAL:
+      case Format::SCRATCH:
+      case Format::EXP:
+      case Format::SOPP:
+      case Format::PSEUDO_BRANCH:
+      case Format::PSEUDO_BARRIER: assert(false);
+      default: return true;
       }
    }
 };
@@ -354,7 +289,8 @@ struct vn_ctx {
     */
    uint32_t exec_id = 1;
 
-   vn_ctx(Program* program_) : program(program_) {
+   vn_ctx(Program* program_) : program(program_)
+   {
       static_assert(sizeof(Temp) == 4, "Temp must fit in 32bits");
       unsigned size = 0;
       for (Block& block : program->blocks)
@@ -363,11 +299,11 @@ struct vn_ctx {
    }
 };
 
-
 /* dominates() returns true if the parent block dominates the child block and
  * if the parent block is part of the same loop or has a smaller loop nest depth.
  */
-bool dominates(vn_ctx& ctx, uint32_t parent, uint32_t child)
+bool
+dominates(vn_ctx& ctx, uint32_t parent, uint32_t child)
 {
    unsigned parent_loop_nest_depth = ctx.program->blocks[parent].loop_nest_depth;
    while (parent < child && parent_loop_nest_depth <= ctx.program->blocks[child].loop_nest_depth)
@@ -376,7 +312,48 @@ bool dominates(vn_ctx& ctx, uint32_t parent, uint32_t child)
    return parent == child;
 }
 
-void process_block(vn_ctx& ctx, Block& block)
+/** Returns whether this instruction can safely be removed
+ *  and replaced by an equal expression.
+ *  This is in particular true for ALU instructions and
+ *  read-only memory instructions.
+ *
+ *  Note that expr_set must not be used with instructions
+ *  which cannot be eliminated.
+ */
+bool
+can_eliminate(aco_ptr<Instruction>& instr)
+{
+   switch (instr->format) {
+   case Format::FLAT:
+   case Format::GLOBAL:
+   case Format::SCRATCH:
+   case Format::EXP:
+   case Format::SOPP:
+   case Format::PSEUDO_BRANCH:
+   case Format::PSEUDO_BARRIER: return false;
+   case Format::DS:
+      return instr->opcode == aco_opcode::ds_bpermute_b32 ||
+             instr->opcode == aco_opcode::ds_permute_b32 ||
+             instr->opcode == aco_opcode::ds_swizzle_b32;
+   case Format::SMEM:
+   case Format::MUBUF:
+   case Format::MIMG:
+   case Format::MTBUF:
+      if (!get_sync_info(instr.get()).can_reorder())
+         return false;
+      break;
+   default: break;
+   }
+
+   if (instr->definitions.empty() || instr->opcode == aco_opcode::p_phi ||
+       instr->opcode == aco_opcode::p_linear_phi || instr->definitions[0].isNoCSE())
+      return false;
+
+   return true;
+}
+
+void
+process_block(vn_ctx& ctx, Block& block)
 {
    std::vector<aco_ptr<Instruction>> new_instructions;
    new_instructions.reserve(block.instructions.size());
@@ -395,14 +372,15 @@ void process_block(vn_ctx& ctx, Block& block)
           instr->opcode == aco_opcode::p_demote_to_helper)
          ctx.exec_id++;
 
-      if (instr->definitions.empty() || is_phi(instr) || instr->definitions[0].isNoCSE()) {
+      if (!can_eliminate(instr)) {
          new_instructions.emplace_back(std::move(instr));
          continue;
       }
 
       /* simple copy-propagation through renaming */
-      bool copy_instr = instr->opcode == aco_opcode::p_parallelcopy ||
-                        (instr->opcode == aco_opcode::p_create_vector && instr->operands.size() == 1);
+      bool copy_instr =
+         instr->opcode == aco_opcode::p_parallelcopy ||
+         (instr->opcode == aco_opcode::p_create_vector && instr->operands.size() == 1);
       if (copy_instr && !instr->definitions[0].isFixed() && instr->operands[0].isTemp() &&
           instr->operands[0].regClass() == instr->definitions[0].regClass()) {
          ctx.renames[instr->definitions[0].tempId()] = instr->operands[0].getTemp();
@@ -445,7 +423,8 @@ void process_block(vn_ctx& ctx, Block& block)
    block.instructions = std::move(new_instructions);
 }
 
-void rename_phi_operands(Block& block, std::map<uint32_t, Temp>& renames)
+void
+rename_phi_operands(Block& block, std::map<uint32_t, Temp>& renames)
 {
    for (aco_ptr<Instruction>& phi : block.instructions) {
       if (phi->opcode != aco_opcode::p_phi && phi->opcode != aco_opcode::p_linear_phi)
@@ -462,8 +441,8 @@ void rename_phi_operands(Block& block, std::map<uint32_t, Temp>& renames)
 }
 } /* end namespace */
 
-
-void value_numbering(Program* program)
+void
+value_numbering(Program* program)
 {
    vn_ctx ctx(program);
    std::vector<unsigned> loop_headers;
@@ -487,10 +466,8 @@ void value_numbering(Program* program)
          rename_phi_operands(block, ctx.renames);
 
       /* increment exec_id when entering nested control flow */
-      if (block.kind & block_kind_branch ||
-          block.kind & block_kind_loop_preheader ||
-          block.kind & block_kind_break ||
-          block.kind & block_kind_continue ||
+      if (block.kind & block_kind_branch || block.kind & block_kind_loop_preheader ||
+          block.kind & block_kind_break || block.kind & block_kind_continue ||
           block.kind & block_kind_discard)
          ctx.exec_id++;
       else if (block.kind & block_kind_continue_or_break)
@@ -504,4 +481,4 @@ void value_numbering(Program* program)
    }
 }
 
-}
+} // namespace aco
