@@ -31,10 +31,9 @@
 #include <assert.h>
 #include "pan_resource.h"
 #include "pan_job.h"
-#include "pan_blend.h"
+#include "pan_blend_cso.h"
 #include "pan_encoder.h"
 #include "pan_texture.h"
-#include "midgard_pack.h"
 
 #include "pipe/p_compiler.h"
 #include "pipe/p_config.h"
@@ -52,18 +51,37 @@
 /* Forward declare to avoid extra header dep */
 struct prim_convert_context;
 
-#define MAX_VARYINGS   4096
-
 #define SET_BIT(lval, bit, cond) \
 	if (cond) \
 		lval |= (bit); \
 	else \
 		lval &= ~(bit);
 
+/* Dirty tracking flags. 3D is for general 3D state. Shader flags are
+ * per-stage. Renderer refers to Renderer State Descriptors. Vertex refers to
+ * vertex attributes/elements. */
+
+enum pan_dirty_3d {
+        PAN_DIRTY_VIEWPORT       = BITFIELD_BIT(0),
+        PAN_DIRTY_SCISSOR        = BITFIELD_BIT(1),
+        PAN_DIRTY_VERTEX         = BITFIELD_BIT(2),
+        PAN_DIRTY_PARAMS         = BITFIELD_BIT(3),
+        PAN_DIRTY_DRAWID         = BITFIELD_BIT(4),
+        PAN_DIRTY_TLS_SIZE       = BITFIELD_BIT(5),
+};
+
+enum pan_dirty_shader {
+        PAN_DIRTY_STAGE_RENDERER = BITFIELD_BIT(0),
+        PAN_DIRTY_STAGE_TEXTURE  = BITFIELD_BIT(1),
+        PAN_DIRTY_STAGE_SAMPLER  = BITFIELD_BIT(2),
+        PAN_DIRTY_STAGE_IMAGE    = BITFIELD_BIT(3),
+        PAN_DIRTY_STAGE_CONST    = BITFIELD_BIT(4),
+        PAN_DIRTY_STAGE_SSBO     = BITFIELD_BIT(5),
+};
+
 struct panfrost_constant_buffer {
         struct pipe_constant_buffer cb[PIPE_MAX_CONSTANT_BUFFERS];
         uint32_t enabled_mask;
-        uint32_t dirty_mask;
 };
 
 struct panfrost_query {
@@ -78,13 +96,13 @@ struct panfrost_query {
         };
 
         /* Memory for the GPU to writeback the value of the query */
-        struct panfrost_bo *bo;
+        struct pipe_resource *rsrc;
 
         /* Whether an occlusion query is for a MSAA framebuffer */
         bool msaa;
 };
 
-struct panfrost_fence {
+struct pipe_fence_handle {
         struct pipe_reference reference;
         uint32_t syncobj;
         bool signaled;
@@ -104,20 +122,28 @@ struct panfrost_context {
         /* Gallium context */
         struct pipe_context base;
 
-        /* Upload manager for small resident GPU-internal data structures, like
-         * sampler descriptors. We use an upload manager since the minimum BO
-         * size from the kernel is 4kb */
-        struct u_upload_mgr *state_uploader;
+        /* Dirty global state */
+        enum pan_dirty_3d dirty;
+
+        /* Per shader stage dirty state */
+        enum pan_dirty_shader dirty_shader[PIPE_SHADER_TYPES];
+
+        /* Unowned pools, so manage yourself. */
+        struct panfrost_pool descs, shaders;
 
         /* Sync obj used to keep track of in-flight jobs. */
         uint32_t syncobj;
 
-        /* Bound job batch and map of panfrost_batch_key to job batches */
-        struct panfrost_batch *batch;
-        struct hash_table *batches;
+        /* Set of 32 batches. When the set is full, the LRU entry (the batch
+         * with the smallest seqnum) is flushed to free a slot.
+         */
+        struct {
+                uint64_t seqnum;
+                struct panfrost_batch slots[PAN_MAX_BATCHES];
+        } batches;
 
-        /* panfrost_bo -> panfrost_bo_access */
-        struct hash_table *accessed_bos;
+        /* Bound job batch */
+        struct panfrost_batch *batch;
 
         /* Within a launch_grid call.. */
         const struct pipe_grid_info *compute_grid;
@@ -133,9 +159,16 @@ struct panfrost_context {
         uint64_t tf_prims_generated;
         struct panfrost_query *occlusion_query;
 
+        bool indirect_draw;
+        unsigned drawid;
         unsigned vertex_count;
         unsigned instance_count;
         unsigned offset_start;
+        unsigned base_vertex;
+        unsigned base_instance;
+        mali_ptr first_vertex_sysval_ptr;
+        mali_ptr base_vertex_sysval_ptr;
+        mali_ptr base_instance_sysval_ptr;
         enum pipe_prim_type active_prim;
 
         /* If instancing is enabled, vertex count padded for instance; if
@@ -156,6 +189,9 @@ struct panfrost_context {
         struct pipe_shader_buffer ssbo[PIPE_SHADER_TYPES][PIPE_MAX_SHADER_BUFFERS];
         uint32_t ssbo_mask[PIPE_SHADER_TYPES];
 
+        struct pipe_image_view images[PIPE_SHADER_TYPES][PIPE_MAX_SHADER_IMAGES];
+        uint32_t image_mask[PIPE_SHADER_TYPES];
+
         struct panfrost_sampler_state *samplers[PIPE_SHADER_TYPES][PIPE_MAX_SAMPLERS];
         unsigned sampler_count[PIPE_SHADER_TYPES];
 
@@ -172,21 +208,44 @@ struct panfrost_context {
         struct pipe_blend_color blend_color;
         struct panfrost_zsa_state *depth_stencil;
         struct pipe_stencil_ref stencil_ref;
-        unsigned sample_mask;
+        uint16_t sample_mask;
         unsigned min_samples;
 
-        struct panfrost_blend_state *blit_blend;
-        struct hash_table *blend_shaders;
-
         struct panfrost_query *cond_query;
         bool cond_cond;
         enum pipe_render_cond_flag cond_mode;
+
+        bool is_noop;
+
+        /* Mask of active render targets */
+        uint8_t fb_rt_mask;
 };
 
 /* Corresponds to the CSO */
 
 struct panfrost_rasterizer {
         struct pipe_rasterizer_state base;
+
+        /* Partially packed RSD words */
+        struct mali_multisample_misc_packed multisample;
+        struct mali_stencil_mask_misc_packed stencil_misc;
+};
+
+/* Linked varyings */
+struct pan_linkage {
+        /* If the upload is owned by the CSO instead
+         * of the pool, the referenced BO. Else,
+         * NULL. */
+        struct panfrost_bo *bo;
+
+        /* Uploaded attribute descriptors */
+        mali_ptr producer, consumer;
+
+        /* Varyings buffers required */
+        uint32_t present;
+
+        /* Per-vertex stride for general varying buffer */
+        uint32_t stride;
 };
 
 /* Variants bundle together to form the backing CSO, bundling multiple
@@ -197,59 +256,26 @@ struct panfrost_shader_state {
         /* Compiled, mapped descriptor, ready for the hardware */
         bool compiled;
 
-        /* Uploaded shader descriptor (TODO: maybe stuff the packed unuploaded
-         * bits in a union to save some memory?) */
+        /* Respectively, shader binary and Renderer State Descriptor */
+        struct panfrost_pool_ref bin, state;
 
-        struct {
-                struct pipe_resource *rsrc;
-                uint32_t offset;
-        } upload;
+        /* For fragment shaders, a prepared (but not uploaded RSD) */
+        struct mali_renderer_state_packed partial_rsd;
 
-        struct MALI_SHADER shader;
-        struct MALI_RENDERER_PROPERTIES properties;
-        struct MALI_PRELOAD preload;
+        struct pan_shader_info info;
 
-        /* Non-descript information */
-        unsigned uniform_count;
-        unsigned work_reg_count;
-        bool can_discard;
-        bool writes_point_size;
-        bool writes_depth;
-        bool writes_stencil;
-        bool reads_point_coord;
-        bool reads_face;
-        bool reads_frag_coord;
-        bool writes_global;
-        unsigned stack_size;
-        unsigned shared_size;
+        /* Linked varyings, for non-separable programs */
+        struct pan_linkage linkage;
 
-        /* Does the fragment shader have side effects? In particular, if output
-         * is masked out, is it legal to skip shader execution? */
-        bool fs_sidefx;
-
-        /* For Bifrost - output type for each RT */
-        enum mali_bifrost_register_file_format blend_types[MALI_BIFROST_BLEND_MAX_RT];
-
-        unsigned attribute_count, varying_count, ubo_count;
-        enum mali_format varyings[PIPE_MAX_ATTRIBS];
-        gl_varying_slot varyings_loc[PIPE_MAX_ATTRIBS];
         struct pipe_stream_output_info stream_output;
         uint64_t so_mask;
 
-        unsigned sysval_count;
-        unsigned sysval[MAX_SYSVAL_COUNT];
-
-        /* Should we enable helper invocations */
-        bool helper_invocations;
-
-        /* GPU-executable memory */
-        struct panfrost_bo *bo;
-
-        BITSET_WORD outputs_read;
+        /* Variants */
         enum pipe_format rt_formats[8];
+        unsigned nr_cbufs;
 
-        /* Blend return addresses */
-        uint32_t blend_ret_addrs[8];
+        /* Mask of state that dirties the sysvals */
+        unsigned dirty_3d, dirty_shader;
 };
 
 /* A collection of varyings (the CSO) */
@@ -273,22 +299,37 @@ struct panfrost_shader_variants {
         unsigned active_variant;
 };
 
+struct pan_vertex_buffer {
+        unsigned vbi;
+        unsigned divisor;
+};
+
 struct panfrost_vertex_state {
         unsigned num_elements;
 
+        /* buffers corresponds to attribute buffer, element_buffers corresponds
+         * to an index in buffers for each vertex element */
+        struct pan_vertex_buffer buffers[PIPE_MAX_ATTRIBS];
+        unsigned element_buffer[PIPE_MAX_ATTRIBS];
+        unsigned nr_bufs;
+
         struct pipe_vertex_element pipe[PIPE_MAX_ATTRIBS];
         unsigned formats[PIPE_MAX_ATTRIBS];
 };
 
 struct panfrost_zsa_state {
         struct pipe_depth_stencil_alpha_state base;
-        enum mali_func alpha_func;
 
-        /* Precomputed stencil state */
-        struct MALI_STENCIL stencil_front;
-        struct MALI_STENCIL stencil_back;
-        u8 stencil_mask_front;
-        u8 stencil_mask_back;
+        /* Is any depth, stencil, or alpha testing enabled? */
+        bool enabled;
+
+        /* Mask of PIPE_CLEAR_{DEPTH,STENCIL} written */
+        unsigned draws;
+
+        /* Prepacked words from the RSD */
+        struct mali_multisample_misc_packed rsd_depth;
+        struct mali_stencil_mask_misc_packed rsd_stencil;
+        struct mali_stencil_packed stencil_front, stencil_back;
 };
 
 struct panfrost_sampler_state {
@@ -300,7 +341,7 @@ struct panfrost_sampler_state {
 
 struct panfrost_sampler_view {
         struct pipe_sampler_view base;
-        struct panfrost_bo *bo;
+        struct panfrost_pool_ref state;
         struct mali_bifrost_texture_packed bifrost_descriptor;
         mali_ptr texture_bo;
         uint64_t modifier;
@@ -346,32 +387,19 @@ panfrost_flush(
         unsigned flags);
 
 bool
-pan_render_condition_check(struct pipe_context *pctx);
-
-mali_ptr panfrost_sfbd_fragment(struct panfrost_batch *batch, bool has_draws);
-mali_ptr panfrost_mfbd_fragment(struct panfrost_batch *batch, bool has_draws);
+panfrost_render_condition_check(struct panfrost_context *ctx);
 
 void
-panfrost_attach_mfbd(struct panfrost_batch *batch, unsigned vertex_count);
-
-void
-panfrost_attach_sfbd(struct panfrost_batch *batch, unsigned vertex_count);
-
-void
-panfrost_emit_midg_tiler(struct panfrost_batch *batch,
-                         struct mali_midgard_tiler_packed *tp,
-                         unsigned vertex_count);
-
-mali_ptr
-panfrost_fragment_job(struct panfrost_batch *batch, bool has_draws);
-
-void
-panfrost_shader_compile(struct panfrost_context *ctx,
+panfrost_shader_compile(struct pipe_screen *pscreen,
+                        struct panfrost_pool *shader_pool,
+                        struct panfrost_pool *desc_pool,
                         enum pipe_shader_ir ir_type,
                         const void *ir,
                         gl_shader_stage stage,
-                        struct panfrost_shader_state *state,
-                        uint64_t *outputs_written);
+                        struct panfrost_shader_state *state);
+
+void
+panfrost_analyze_sysvals(struct panfrost_shader_state *ss);
 
 void
 panfrost_create_sampler_view_bo(struct panfrost_sampler_view *so,
@@ -388,5 +416,27 @@ panfrost_vertex_buffer_address(struct panfrost_context *ctx, unsigned i);
 void
 panfrost_compute_context_init(struct pipe_context *pctx);
 
+static inline void
+panfrost_dirty_state_all(struct panfrost_context *ctx)
+{
+        ctx->dirty = ~0;
+
+        for (unsigned i = 0; i < PIPE_SHADER_TYPES; ++i)
+                ctx->dirty_shader[i] = ~0;
+}
+
+static inline void
+panfrost_clean_state_3d(struct panfrost_context *ctx)
+{
+        ctx->dirty = 0;
+
+        for (unsigned i = 0; i < PIPE_SHADER_TYPES; ++i) {
+                if (i != PIPE_SHADER_COMPUTE)
+                        ctx->dirty_shader[i] = 0;
+        }
+}
+
+void
+panfrost_cmdstream_context_init(struct pipe_context *pipe);
 
 #endif
