@@ -34,6 +34,7 @@
 
 #include <X11/Xlib-xcb.h>
 
+#include "loader_dri_helper.h"
 #include "loader_dri3_helper.h"
 #include "util/macros.h"
 #include "drm-uapi/drm_fourcc.h"
@@ -359,6 +360,9 @@ loader_dri3_drawable_fini(struct loader_dri3_drawable *draw)
       xcb_unregister_for_special_event(draw->conn, draw->special_event);
    }
 
+   if (draw->region)
+      xcb_xfixes_destroy_region(draw->conn, draw->region);
+
    cnd_destroy(&draw->event_cnd);
    mtx_destroy(&draw->mtx);
 }
@@ -384,6 +388,7 @@ loader_dri3_drawable_init(xcb_connection_t *conn,
    draw->ext = ext;
    draw->vtable = vtable;
    draw->drawable = drawable;
+   draw->region = 0;
    draw->dri_screen = dri_screen;
    draw->is_different_gpu = is_different_gpu;
    draw->multiplanes_available = multiplanes_available;
@@ -1009,7 +1014,7 @@ loader_dri3_swap_buffers_msc(struct loader_dri3_drawable *draw,
        */
       ++draw->send_sbc;
       if (target_msc == 0 && divisor == 0 && remainder == 0)
-         target_msc = draw->msc + draw->swap_interval *
+         target_msc = draw->msc + abs(draw->swap_interval) *
                       (draw->send_sbc - draw->recv_sbc);
       else if (divisor == 0 && remainder > 0) {
          /* From the GLX_OML_sync_control spec:
@@ -1029,11 +1034,19 @@ loader_dri3_swap_buffers_msc(struct loader_dri3_drawable *draw,
        *     "If <interval> is set to a value of 0, buffer swaps are not
        *      synchronized to a video frame."
        *
+       * From GLX_EXT_swap_control_tear:
+       *
+       *     "If <interval> is negative, the minimum number of video frames
+       *      between buffer swaps is the absolute value of <interval>. In this
+       *      case, if abs(<interval>) video frames have already passed from
+       *      the previous swap when the swap is ready to be performed, the
+       *      swap will occur without synchronization to a video frame."
+       *
        * Implementation note: It is possible to enable triple buffering
        * behaviour by not using XCB_PRESENT_OPTION_ASYNC, but this should not be
        * the default.
        */
-      if (draw->swap_interval == 0)
+      if (draw->swap_interval <= 0)
           options |= XCB_PRESENT_OPTION_ASYNC;
 
       /* If we need to populate the new back, but need to reuse the back
@@ -1049,6 +1062,11 @@ loader_dri3_swap_buffers_msc(struct loader_dri3_drawable *draw,
       back->busy = 1;
       back->last_swap = draw->send_sbc;
 
+      if (!draw->region) {
+         draw->region = xcb_generate_id(draw->conn);
+         xcb_xfixes_create_region(draw->conn, draw->region, 0, NULL);
+      }
+
       xcb_xfixes_region_t region = 0;
       xcb_rectangle_t xcb_rects[64];
 
@@ -1061,8 +1079,8 @@ loader_dri3_swap_buffers_msc(struct loader_dri3_drawable *draw,
             xcb_rects[i].height = rect[3];
          }
 
-         region = xcb_generate_id(draw->conn);
-         xcb_xfixes_create_region(draw->conn, region, n_rects, xcb_rects);
+         region = draw->region;
+         xcb_xfixes_set_region(draw->conn, region, n_rects, xcb_rects);
       }
 
       xcb_present_pixmap(draw->conn,
@@ -1082,9 +1100,6 @@ loader_dri3_swap_buffers_msc(struct loader_dri3_drawable *draw,
                          remainder, 0, NULL);
       ret = (int64_t) draw->send_sbc;
 
-      if (region)
-         xcb_xfixes_destroy_region(draw->conn, region);
-
       /* Schedule a server-side back-preserving blit if necessary.
        * This happens iff all conditions below are satisfied:
        * a) We have a fake front,
@@ -1141,6 +1156,8 @@ loader_dri3_open(xcb_connection_t *conn,
 {
    xcb_dri3_open_cookie_t       cookie;
    xcb_dri3_open_reply_t        *reply;
+   xcb_xfixes_query_version_cookie_t fixes_cookie;
+   xcb_xfixes_query_version_reply_t *fixes_reply;
    int                          fd;
 
    cookie = xcb_dri3_open(conn,
@@ -1160,6 +1177,13 @@ loader_dri3_open(xcb_connection_t *conn,
    free(reply);
    fcntl(fd, F_SETFD, fcntl(fd, F_GETFD) | FD_CLOEXEC);
 
+   /* let the server know our xfixes level */
+   fixes_cookie = xcb_xfixes_query_version(conn,
+                                           XCB_XFIXES_MAJOR_VERSION,
+                                           XCB_XFIXES_MINOR_VERSION);
+   fixes_reply = xcb_xfixes_query_version_reply(conn, fixes_cookie, NULL);
+   free(fixes_reply);
+
    return fd;
 }
 
@@ -1300,12 +1324,14 @@ dri3_alloc_render_buffer(struct loader_dri3_drawable *draw, unsigned int format,
                          int width, int height, int depth)
 {
    struct loader_dri3_buffer *buffer;
-   __DRIimage *pixmap_buffer;
+   __DRIimage *pixmap_buffer = NULL, *linear_buffer_display_gpu = NULL;
    xcb_pixmap_t pixmap;
    xcb_sync_fence_t sync_fence;
    struct xshmfence *shm_fence;
    int buffer_fds[4], fence_fd;
    int num_planes = 0;
+   uint64_t *modifiers = NULL;
+   uint32_t count = 0;
    int i, mod;
    int ret;
 
@@ -1340,8 +1366,6 @@ dri3_alloc_render_buffer(struct loader_dri3_drawable *draw, unsigned int format,
          xcb_dri3_get_supported_modifiers_cookie_t mod_cookie;
          xcb_dri3_get_supported_modifiers_reply_t *mod_reply;
          xcb_generic_error_t *error = NULL;
-         uint64_t *modifiers = NULL;
-         uint32_t count = 0;
 
          mod_cookie = xcb_dri3_get_supported_modifiers(draw->conn,
                                                        draw->window,
@@ -1387,32 +1411,17 @@ dri3_alloc_render_buffer(struct loader_dri3_drawable *draw, unsigned int format,
          }
 
          free(mod_reply);
-
-         /* don't use createImageWithModifiers() if we have no
-          * modifiers, other things depend on the use flags when
-          * there are no modifiers to know that a buffer can be
-          * shared.
-          */
-         if (modifiers) {
-            buffer->image = draw->ext->image->createImageWithModifiers(draw->dri_screen,
-                                                                       width, height,
-                                                                       format,
-                                                                       modifiers,
-                                                                       count,
-                                                                       buffer);
-         }
-
-         free(modifiers);
       }
 #endif
-      if (!buffer->image)
-         buffer->image = draw->ext->image->createImage(draw->dri_screen,
-                                                       width, height,
-                                                       format,
-                                                       __DRI_IMAGE_USE_SHARE |
-                                                       __DRI_IMAGE_USE_SCANOUT |
-                                                       __DRI_IMAGE_USE_BACKBUFFER,
-                                                       buffer);
+      buffer->image = loader_dri_create_image(draw->dri_screen, draw->ext->image,
+                                              width, height, format,
+                                              __DRI_IMAGE_USE_SHARE |
+                                              __DRI_IMAGE_USE_SCANOUT |
+                                              __DRI_IMAGE_USE_BACKBUFFER |
+                                              (draw->is_protected_content ?
+                                               __DRI_IMAGE_USE_PROTECTED : 0),
+                                              modifiers, count, buffer);
+      free(modifiers);
 
       pixmap_buffer = buffer->image;
 
@@ -1428,18 +1437,39 @@ dri3_alloc_render_buffer(struct loader_dri3_drawable *draw, unsigned int format,
       if (!buffer->image)
          goto no_image;
 
-      buffer->linear_buffer =
-        draw->ext->image->createImage(draw->dri_screen,
-                                      width, height,
-                                      dri3_linear_format_for_format(draw, format),
-                                      __DRI_IMAGE_USE_SHARE |
-                                      __DRI_IMAGE_USE_LINEAR |
-                                      __DRI_IMAGE_USE_BACKBUFFER,
-                                      buffer);
-      pixmap_buffer = buffer->linear_buffer;
+      /* if driver name is same only then dri_screen_display_gpu is set.
+       * This check is needed because for simplicity render gpu image extension
+       * is also used for display gpu.
+       */
+      if (draw->dri_screen_display_gpu) {
+         linear_buffer_display_gpu =
+           draw->ext->image->createImage(draw->dri_screen_display_gpu,
+                                         width, height,
+                                         dri3_linear_format_for_format(draw, format),
+                                         __DRI_IMAGE_USE_SHARE |
+                                         __DRI_IMAGE_USE_LINEAR |
+                                         __DRI_IMAGE_USE_BACKBUFFER |
+                                         __DRI_IMAGE_USE_SCANOUT,
+                                         buffer);
+         pixmap_buffer = linear_buffer_display_gpu;
+      }
 
-      if (!buffer->linear_buffer)
-         goto no_linear_buffer;
+      if (!pixmap_buffer) {
+         buffer->linear_buffer =
+           draw->ext->image->createImage(draw->dri_screen,
+                                         width, height,
+                                         dri3_linear_format_for_format(draw, format),
+                                         __DRI_IMAGE_USE_SHARE |
+                                         __DRI_IMAGE_USE_LINEAR |
+                                         __DRI_IMAGE_USE_BACKBUFFER |
+                                         __DRI_IMAGE_USE_SCANOUT,
+                                         buffer);
+
+         pixmap_buffer = buffer->linear_buffer;
+         if (!buffer->linear_buffer) {
+            goto no_linear_buffer;
+         }
+      }
    }
 
    /* X want some information about the planes, so ask the image for it
@@ -1481,6 +1511,26 @@ dri3_alloc_render_buffer(struct loader_dri3_drawable *draw, unsigned int format,
    if (!ret)
       buffer->modifier = DRM_FORMAT_MOD_INVALID;
 
+   if (draw->is_different_gpu && draw->dri_screen_display_gpu &&
+       linear_buffer_display_gpu) {
+      /* The linear buffer was created in the display GPU's vram, so we
+       * need to make it visible to render GPU
+       */
+      buffer->linear_buffer =
+         draw->ext->image->createImageFromFds(draw->dri_screen,
+                                              width,
+                                              height,
+                                              image_format_to_fourcc(format),
+                                              &buffer_fds[0], num_planes,
+                                              &buffer->strides[0],
+                                              &buffer->offsets[0],
+                                              buffer);
+      if (!buffer->linear_buffer)
+         goto no_buffer_attrib;
+
+      draw->ext->image->destroyImage(linear_buffer_display_gpu);
+   }
+
    pixmap = xcb_generate_id(draw->conn);
 #ifdef HAVE_DRI3_MODIFIERS
    if (draw->multiplanes_available &&
@@ -2007,6 +2057,9 @@ dri3_free_buffers(__DRIdrawable *driDrawable,
       first_id = LOADER_DRI3_FRONT_ID;
       /* Don't free a fake front holding new backbuffer content. */
       n_id = (draw->cur_blit_source == LOADER_DRI3_FRONT_ID) ? 0 : 1;
+      break;
+   default:
+      unreachable("unhandled buffer_type");
    }
 
    for (buf_id = first_id; buf_id < first_id + n_id; buf_id++) {
